"use strict";
/*
 * This program and the accompanying materials are made available under the terms of the
 * Eclipse Public License v2.0 which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-v20.html
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Copyright Contributors to the Zowe Project.
 *
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FTPConfig = void 0;
const util_1 = require("util");
const imperative_1 = require("@zowe/imperative");
const ftpConnectionOptionGroup = "FTP Connection options";
const tlsConnectionOptionGroup = "TLS / Secure Connection options";
class FTPConfig {
    /**
     * Convert a profile into a config object used to connect to
     * zos-node-accessor
     * @param arguments - the arguments passed by the user
     * @param doPrompting - Whether to prompt for missing arguments (defaults to true)
     * @param {IHandlerParameters} handlerParams - The command parameters object for daemon mode prompting
     * @returns  the connection to zos-node-accessor's APIs
     */
    static connectFromArguments(args, doPrompting = true, handlerParams) {
        return __awaiter(this, void 0, void 0, function* () {
            const sessCfg = {
                type: "basic",
                hostname: args.host,
                port: args.port,
                user: args.user,
                password: args.password
            };
            const sessCfgWithCreds = yield imperative_1.ConnectionPropsForSessCfg.addPropsOrPrompt(sessCfg, args, { doPrompting, parms: handlerParams });
            const ftpConfig = FTPConfig.createConfigFromArguments(Object.assign(Object.assign({}, args), { host: sessCfgWithCreds.hostname, port: sessCfgWithCreds.port, user: sessCfgWithCreds.user, password: sessCfgWithCreds.password }));
            const zosAccessor = new (require("zos-node-accessor"))();
            return zosAccessor.connect(ftpConfig);
        });
    }
    /**
     * Convert a profile into a config object used to connect to
     * zos-node-accessor
     * @param arguments - the profile created by the user
     * @returns the config object for zos-node-accessor's ftp connection
     */
    static createConfigFromArguments(args) {
        // build the options argument for zos-node-accessor
        const result = {
            host: args.host,
            user: args.user,
            password: args.password,
            port: args.port,
            connTimeout: args.connectionTimeout
        };
        if (args.secureFtp != null) {
            if ((0, util_1.isString)(args.secureFtp) && args.secureFtp.trim().toLowerCase() === "true") {
                result.secure = true;
            }
            else {
                result.secure = args.secureFtp;
            }
        }
        if (this.profileHasSecureOptions(args)) {
            result.secureOptions = {
                rejectUnauthorized: args.rejectUnauthorized,
                serverName: args.serverName,
            };
        }
        return result;
    }
    /**
     * Determine whether the user specified any TLS connection options
     * @param args - the profile loaded from the user
     * @returns true if the user has specified any secure connection options
     */
    static profileHasSecureOptions(args) {
        return (args.rejectUnauthorized != null) || (args.serverName !== null);
    }
}
exports.FTPConfig = FTPConfig;
FTPConfig.OPTION_HOST = {
    type: "string",
    name: "host", aliases: ["H"],
    description: "The hostname or IP address of the z/OS server to connect to.",
    group: ftpConnectionOptionGroup
};
FTPConfig.OPTION_PORT = {
    type: "number",
    name: "port", aliases: ["P"],
    description: "The port of the z/OS FTP server.",
    defaultValue: 21,
    group: ftpConnectionOptionGroup
};
FTPConfig.OPTION_USER = {
    type: "string",
    name: "user", aliases: ["u"],
    description: "Username for authentication on z/OS",
    group: ftpConnectionOptionGroup
};
FTPConfig.OPTION_PASSWORD = {
    type: "string",
    name: "password", aliases: ["p", "pass", "pw"],
    description: "Password to authenticate to FTP.",
    group: ftpConnectionOptionGroup
};
FTPConfig.OPTION_SECURE_FTP = {
    name: "secure-ftp",
    type: "boolean",
    defaultValue: true,
    description: "Set to true for both control and data connection encryption," +
        " 'control' for control connection encryption only, or 'implicit' for implicitly" +
        " encrypted control connection (this mode is deprecated in modern times, but usually uses port 990). " +
        "Note: Unfortunately, this plugin's functionality only works with FTP and FTPS, not 'SFTP' which is FTP over SSH.",
    group: ftpConnectionOptionGroup
};
FTPConfig.OPTION_REJECT_UNAUTHORIZED = {
    name: "reject-unauthorized",
    aliases: ["ru"],
    description: "Reject self-signed certificates. Only specify this if " +
        "you are connecting to a secure FTP instance.",
    type: "boolean",
    defaultValue: null,
    group: tlsConnectionOptionGroup
};
FTPConfig.OPTION_SERVER_NAME = {
    name: "server-name",
    aliases: ["sn"],
    description: "Server name for the SNI (Server Name Indication) TLS extension. " +
        "Only specify if you are connecting securely",
    group: tlsConnectionOptionGroup,
    type: "string",
    defaultValue: null
};
FTPConfig.OPTION_CONNECTION_TIMEOUT = {
    name: "connection-timeout", aliases: ["ct"],
    description: "How long (in milliseconds) to wait for the control connection to be established.",
    defaultValue: 10000,
    type: "number",
    group: ftpConnectionOptionGroup
};
FTPConfig.OPTION_ENCODING = {
    name: "encoding", aliases: ["ec"],
    description: "The encoding for download and upload of z/OS data set.",
    defaultValue: null,
    type: "string",
    group: ftpConnectionOptionGroup
};
/**
 * Shared options used by all FTP commands
 */
FTPConfig.FTP_CONNECTION_OPTIONS = [
    FTPConfig.OPTION_HOST,
    FTPConfig.OPTION_PORT,
    FTPConfig.OPTION_USER,
    FTPConfig.OPTION_PASSWORD,
    FTPConfig.OPTION_SECURE_FTP,
    FTPConfig.OPTION_REJECT_UNAUTHORIZED,
    FTPConfig.OPTION_SERVER_NAME,
    FTPConfig.OPTION_CONNECTION_TIMEOUT
];
//# sourceMappingURL=FTPConfig.js.map