"use strict";
/*
 * This program and the accompanying materials are made available under the terms of the
 * Eclipse Public License v2.0 which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-v20.html
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Copyright Contributors to the Zowe Project.
 *
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UssUtils = void 0;
const PATH = require("path");
const fs = require("fs");
const imperative_1 = require("@zowe/imperative");
const CoreUtils_1 = require("./CoreUtils");
const StreamUtils_1 = require("./StreamUtils");
class UssUtils {
    /**
     * Lists files under the specified directory.
     *
     * @param connection - zos-node-accessor connection
     * @param directory - directory to list
     * @returns file entries
     */
    static listFiles(connection, directory) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug("Listing USS files in the directory '%s'", directory);
            // Only support wildcard matching in file names as follows.
            //     "/dir1/dir2"
            //     "/dir1/dir2/file*"       => /^file.*$/
            //     "/dir1/dir2/*suffix"     => /^.*suffix$/
            //     "/dir1/dir2/file*suffix" => /^file.*suffix$/
            let directoryToList = directory;
            const slashPosn = directory.lastIndexOf("/");
            let filter;
            if (slashPosn !== -1) {
                const lastPart = directory.substring(slashPosn + 1);
                if (lastPart.indexOf("*") !== -1) {
                    directoryToList = directory.substring(0, slashPosn);
                    const pattern = "^" + lastPart.replace(/\*/g, ".*") + "$";
                    filter = (fileName) => (fileName.match(pattern) != null);
                    this.log.debug("Listing USS files in the directory '%s' with pattern '%s'", directoryToList, pattern);
                }
            }
            let files = yield connection.listDataset(directoryToList);
            if (filter) {
                files = files.filter((file) => filter(file.name));
            }
            this.log.debug("Found %d matching files", files.length);
            const filteredFiles = files.map((file) => CoreUtils_1.CoreUtils.addLowerCaseKeysToObject(file));
            return filteredFiles;
        });
    }
    /**
     * Makes directory.
     *
     * @param connection - zos-node-accessor connection
     * @param directory - directory to make
     */
    static makeDirectory(connection, directory) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug("Attempting to make USS directory '%s'", directory);
            yield connection.makeDirectory(directory);
        });
    }
    /**
     * Renames file or directory.
     *
     * @param connection - zos-node-accessor connection
     * @param oldNanme - old name of file or directory
     * @param newName - new name of file or directory
     */
    static renameFile(connection, oldNanme, newName) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug("Attempting to rename USS file or directory from '%s' to '%s'", oldNanme, newName);
            yield connection.rename(oldNanme, newName);
        });
    }
    /**
     * Deletes file or directory.
     *
     * @param connection - zos-node-accessor connection
     * @param fileOrDir - file or directory
     * @param option - delete file option
     */
    static deleteFile(connection, fileOrDir, option) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug("Deleting USS file '%s'", fileOrDir);
            if (option && option.recursive) {
                yield UssUtils.deleteDirectory(connection, fileOrDir, option.console);
            }
            else {
                yield connection.deleteDataset(fileOrDir);
            }
        });
    }
    /**
     * Downloads file.
     *
     * @param connection - zos-node-accessor connection
     * @param ussFile - file path name
     * @param option - download option
     * @returns promise to return buffer when accomplished. If `localFile` is specified, return undefined.
     */
    static downloadFile(connection, ussFile, option) {
        return __awaiter(this, void 0, void 0, function* () {
            const transferType = option.transferType || CoreUtils_1.TRANSFER_TYPE_ASCII;
            let buffer;
            let length;
            const stream = yield connection.getDataset(ussFile, transferType, true);
            if (option.localFile) {
                this.log.debug("Downloading USS file '%s' to local file '%s' in transfer mode '%s", ussFile, option.localFile, transferType);
                imperative_1.IO.createDirsSyncFromFilePath(option.localFile);
                const writable = fs.createWriteStream(option.localFile);
                length = yield StreamUtils_1.StreamUtils.streamToStream(option.size, stream, writable, option.progress);
            }
            else {
                this.log.debug("Downloading USS file '%s' in transfer mode '%s", ussFile, transferType);
                buffer = yield StreamUtils_1.StreamUtils.streamToBuffer(option.size, stream, option.progress);
                length = buffer.byteLength;
            }
            this.log.info("Successfully downloaded %d bytes of content from %s", length, ussFile);
            return Promise.resolve(buffer);
        });
    }
    /**
     * Uploads file.
     *
     * @param connection - zos-node-accessor connection
     * @param ussFile - file path name
     * @param option - upload option
     */
    static uploadFile(connection, ussFile, option) {
        return __awaiter(this, void 0, void 0, function* () {
            const transferType = option.transferType || CoreUtils_1.TRANSFER_TYPE_ASCII;
            let content = option.content;
            if (option.localFile) {
                this.log.debug("Attempting to upload from local file '%s' to USS file '%s' in transfer mode '%s'", option.localFile, ussFile, transferType);
                content = imperative_1.IO.readFileSync(option.localFile, undefined, transferType === "binary");
            }
            else {
                this.log.debug("Attempting to upload to USS file'%s' in transfer mode '%s'", ussFile, transferType);
            }
            if (transferType === CoreUtils_1.TRANSFER_TYPE_ASCII) {
                // if we're in ascii mode, we need carriage returns to avoid errors
                content = Buffer.from(CoreUtils_1.CoreUtils.addCarriageReturns(content.toString()));
            }
            yield connection.uploadDataset(content, ussFile, transferType);
        });
    }
    /**
     * Try to make sure a unix path conforms to what ftp expects
     * Includes fixing paths that are messed up by git for windows tools
     * like turning paths /u/users into U:/Users
     * @param path - the path to normalize
     * @returns the normalized path
     */
    static normalizeUnixPath(path) {
        this.log.debug("Normalizing unix path %s", path);
        // let's fix windows-ported unix tools trying to be helpful by converting paths to
        // windows style paths
        path = path.trim();
        const windowsStylePathPattern = /^([A-Z]+):\//i;
        const doubleSlashPattern = /^\/\//g;
        if (windowsStylePathPattern.test(path)) {
            const normalizedPath = path.replace(windowsStylePathPattern, (overallMatch, driveLetter) => {
                return "/" + driveLetter.toLowerCase() + "/";
            });
            this.log.debug("Path has been normalized to %s", normalizedPath);
            return normalizedPath;
        }
        else if (doubleSlashPattern.test(path)) {
            // if the path starts with two slashes to avoid the 'correction'
            // then we will remove one of the slashes
            const normalizedPath = path.replace(doubleSlashPattern, "/");
            this.log.debug("Users specified two slashes at the beginning of the path, " +
                "possibly to avoid MSYS path correction. Replacing two slashes with one. Result: %s", normalizedPath);
            return normalizedPath;
        }
        else {
            this.log.debug("Path was already normalized. Using the path as-is.");
            return path;
        }
    }
    static deleteDirectory(connection, dir, response) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield connection.listDataset(dir);
            for (const file of files) {
                const filePath = PATH.posix.join(dir, file.name);
                if (file.isDirectory) {
                    yield this.deleteDirectory(connection, filePath, response);
                }
                else {
                    yield connection.deleteDataset(filePath);
                    if (response) {
                        response.log("Deleted %s", filePath);
                    }
                }
            }
            yield connection.deleteDataset(dir);
            if (response) {
                response.log("Deleted %s", dir);
            }
        });
    }
    static checkAbsoluteFilePath(filePath) {
        if (!filePath.startsWith('/')) {
            throw new imperative_1.ImperativeError({ msg: "Please check the uss file path. The full file path is required." });
        }
        else {
            return filePath;
        }
    }
    static get log() {
        return imperative_1.Logger.getAppLogger();
    }
}
exports.UssUtils = UssUtils;
//# sourceMappingURL=UssUtils.js.map