"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopRegion = exports.stopTransaction = exports.stopProgram = void 0;
const imperative_1 = require("@zowe/imperative");
const rest_1 = require("../../rest");
const constants_1 = require("../../constants");
const CommonUtils_1 = require("../CommonUtils");
const ImsSessionUtils_1 = require("../../../cli/ImsSessionUtils");
/**
 * Stop program in IMS through REST API
 * @param {ImsSession} session - the session to connect to IMS with
 * @param {IUpdateProgramParms} parms - parameters for stopping a program
 * @returns {Promise<IIMSApiResponse>} promise that resolves to the response (XML parsed into a javascript object)
 *                          when the request is complete
 * @throws {ImperativeError} IMS program name not defined or blank
 * @throws {ImperativeError} ImsRestClient request fails
 */
function stopProgram(session, parms) {
    return __awaiter(this, void 0, void 0, function* () {
        imperative_1.ImperativeExpect.toNotBeNullOrUndefined(parms.name, "IMS program name is required");
        imperative_1.ImperativeExpect.toBeDefinedAndNonBlank(parms.name[0], "IMS program name", "IMS program name is required");
        imperative_1.Logger.getAppLogger().debug("Attempting to stop programs(s) with the following parameters:\n%s", JSON.stringify(parms));
        let resource = ImsSessionUtils_1.ImsSessionUtils.getUrl(session.ISession.basePath) + session.plex + "/" + constants_1.ImsConstants.PROGRAM;
        resource = resource + CommonUtils_1.getQueryFromParms(parms, { stop: "SCHD" });
        return rest_1.ImsRestClient.putExpectJSON(session, resource, [], undefined);
    });
}
exports.stopProgram = stopProgram;
/**
 * Stop transaction in IMS through REST API
 * @param {ImsSession} session - the session to connect to IMS with
 * @param {IUpdateTransactionParms} parms - parameters for stopping a transaction
 * @returns {Promise<IIMSApiResponse>} promise that resolves to the response (XML parsed into a javascript object)
 *                          when the request is complete
 * @throws {ImperativeError} IMS program name not defined or blank
 * @throws {ImperativeError} ImsRestClient request fails
 */
function stopTransaction(session, parms) {
    return __awaiter(this, void 0, void 0, function* () {
        imperative_1.ImperativeExpect.toNotBeNullOrUndefined(parms.name, "IMS transaction name is required");
        imperative_1.ImperativeExpect.toBeDefinedAndNonBlank(parms.name[0], "IMS Transaction name", "IMS transaction name is required");
        imperative_1.Logger.getAppLogger().debug("Attempting to stop transactions(s) with the following parameters:\n%s", JSON.stringify(parms));
        let resource = ImsSessionUtils_1.ImsSessionUtils.getUrl(session.ISession.basePath) + session.plex + "/" + constants_1.ImsConstants.TRANSACTION;
        resource = resource + CommonUtils_1.getQueryFromParms(parms, { stop: "SCHD" });
        return rest_1.ImsRestClient.putExpectJSON(session, resource, [], undefined);
    });
}
exports.stopTransaction = stopTransaction;
/**
 * Stop region in IMS through REST API
 * @param {ImsSession} session - the session to connect to IMS with
 * @param {IStopRegionParms} parms - parameters for stopping the region
 * @returns {Promise<IIMSApiResponse>} promise that resolves to the response (XML parsed into a javascript object)
 *                          when the request is complete
 * @throws {ImperativeError} IMS program name not defined or blank
 * @throws {ImperativeError} ImsRestClient request fails
 */
function stopRegion(session, parms) {
    return __awaiter(this, void 0, void 0, function* () {
        imperative_1.ImperativeExpect.toBeEqual(parms.reg_num == null && parms.job_name == null, false, "Either region number or job name (but not both) must be specified.");
        imperative_1.ImperativeExpect.toBeEqual(parms.reg_num != null && parms.job_name != null, false, "Either region number or job name (but not both) must be specified.");
        if (parms.reg_num == null) {
            imperative_1.ImperativeExpect.toBeDefinedAndNonBlank(parms.job_name, "If job name is specified it must have a value.");
        }
        imperative_1.Logger.getAppLogger().debug("Attempting to stop a region with the following parameters:\n%s", JSON.stringify(parms));
        let resource = ImsSessionUtils_1.ImsSessionUtils.getUrl(session.ISession.basePath) + session.plex + "/" + constants_1.ImsConstants.REGION + "/" + constants_1.ImsConstants.STOP;
        resource = resource + CommonUtils_1.getQueryFromParms(parms);
        return rest_1.ImsRestClient.putExpectJSON(session, resource, [], undefined);
    });
}
exports.stopRegion = stopRegion;
//# sourceMappingURL=Stop.js.map